Morph subclass: #ScaleMorph2
	instanceVariableNames: 'min max minorTick subMajorTickFactor majorTickFactor labels majorTicks majorMin majorMax subMajorTick majorTick labelBlock captionMorph captionMorphWidth captionMorphHeight'
	classVariableNames: 'ClipInset LabelColor MajorTickSize MinorTickSize SubMajorTickSize TickColor'
	poolDictionaries: ''
	category: 'ScaleMorph2'!
!ScaleMorph2 commentStamp: 'mk 2/25/2004 17:47' prior: 0!
This is an abstract class with subclasses whose instances are similar to the ScaleMorph.

There are three kind of ticks:

- minor tick ("1 tick")
- submajor tick ("5 tick")
- major tick ("10 tick")

Instance variables

min					--	Minimal coordinate this scale is able to show.
max					--	Maximal coordinate this scale is able to show.
minorTick			--	If the coordinate is divisible by this value, it means that
						its minor tick should be displayed. Default value 1.
subMajorTickFactor	--	If the coordinate is divisible by minorTick * subMajorTickFactor,
						it means that its sub-major tick should be displayed.
						Default value is: 5.
majorTickFactor		--	If the coordinate is divisible by
						minorTick * subMajorTickFactor * majorTickFactor, it means
						that its major tick should be displayed.
						Default value: 10
labels majorTicks majorMinX majorMaxX subMajorTick majorTick
					--	Various auxiliary instance variables set up in
						updateLabels method.
labelBlock			--	A block with one parameter which should return string representation
						of a given major tick.
						Default value is
							[:i | i printString]
						i.e. returns the major tick itself. Other possible values:
							[:i | i asFloat printString]
captionMorph		--	String set up by caption: method.

Class variables affecting the drawOn: operation

ClipInset				--	We clip the draw operations to the rectangle which is equivalent
							to the original bouds of this morph inset by this value.
MinorTickSize			--	Size of the minor ticks (in the world coordinate system)
SubMajorTickSize		--	Size of the sub-major tick (in the world coordinate system)
MajorTickSize			--	Size of the major ticks (in the world coordinate system)
VerticalLabelSkip		--	Vertical space between the bottom of the major tick and the
							number labeling particular major tick.
LabelColor				--	Color of the labeling text (numbers on the scale and the
							describing text).
TickColor				--	Color of the tick lines.
VerticalCaptionSkip		--	Vertical space between the bottom of this scale and the caption.Main comment stating the purpose of this class and relevant relationship to other classes.!
]style[(77 10 863 6 12 309 8 883)f1,f1LScaleMorph Comment;,f1,f1LPlotMorph2HorizontalScale updateLabels;,f1b,f1,f1b,f1!


!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/25/2004 00:04'!
caption
	self subclassResponsibility! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/26/2004 12:02'!
labelBlock: aBlock
	labelBlock _ aBlock.
	self changed! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/25/2004 15:11'!
majorTickFactor
	^ majorTickFactor! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 4/30/2005 17:52'!
majorTickFactor: aNumber
	[(0 < aNumber) & aNumber isInteger] assert.

	majorTickFactor _ aNumber.
	self changed! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/25/2004 15:10'!
max
	^ max! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/26/2004 12:02'!
max: aNumber
	[min < aNumber & (aNumber isInteger | aNumber isFraction)] assert.

	max _ aNumber.
	self changed! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/25/2004 15:10'!
min
	^ min! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/26/2004 12:02'!
min: aNumber
	[aNumber < max & (aNumber isInteger | aNumber isFraction)] assert.

	min _ aNumber.
	self changed! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/26/2004 12:02'!
min: aNumber1 max: aNumber2
	[aNumber1 < aNumber2 & (aNumber1 isInteger | aNumber1 isFraction)
	                            & (aNumber2 isInteger | aNumber2 isFraction)] assert.

	min _ aNumber1.
	max _ aNumber2.
	self changed! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/25/2004 15:11'!
minorTick
	^ minorTick! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/26/2004 12:02'!
minorTick: aNumber
	[(0 < aNumber) & (aNumber isInteger | aNumber isFraction)] assert.

	minorTick _ aNumber.
	self changed! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 4/30/2005 12:14'!
size
	"Actually, the number of the spaces between the ticks."

	^ max - min! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/25/2004 15:11'!
subMajorTickFactor
	^ subMajorTickFactor! !

!ScaleMorph2 methodsFor: 'accessing' stamp: 'mk 4/30/2005 17:53'!
subMajorTickFactor: aNumber
	[(0 < aNumber) & (aNumber isInteger | aNumber isFraction)] assert.

	subMajorTickFactor _ aNumber.
	self changed! !


!ScaleMorph2 methodsFor: 'initialization' stamp: 'mk 2/26/2004 11:52'!
initialize
	super initialize.
	
	min _ 0.
	max _ 100.
	minorTick _ 1.
	subMajorTickFactor _ 5.
	majorTickFactor _ 2.
	self color: Color transparent.
	labelBlock _ [:i | i asString].! !


!ScaleMorph2 methodsFor: 'menu' stamp: 'mk 2/25/2004 00:03'!
changeCaption
	self subclassResponsibility! !

!ScaleMorph2 methodsFor: 'menu' stamp: 'mk 4/30/2005 17:52'!
changeMajorTickFactor
	"We expect an integer which is not less than 2."
	| string number convertBlock |
	[number isNil]
		whileTrue:
			[
				string _ FillInTheBlank
					request: 'Enter new major tick factor value.'
					initialAnswer: majorTickFactor asString.
				convertBlock _ [number _ string asNumber asInteger].
				convertBlock on: Error do: [].
					string = ''
					ifTrue:
						[
							"User pressed cancel."
							^ self
						].
				number < 1
					ifTrue: [number _ nil]
			].
			
	self majorTickFactor: number! !

!ScaleMorph2 methodsFor: 'menu' stamp: 'mk 2/24/2004 23:33'!
changeMax
	"We expect an integer which is greater than minX."
	| string number convertBlock |
	[number isNil]
		whileTrue:
			[
				string _ FillInTheBlank
					request: 'Enter new minimal value.'
					initialAnswer: max asString.
				convertBlock _ [number _ string asNumber asInteger].
				convertBlock on: Error do: [].
				string = ''
					ifTrue:
						[
							"User pressed cancel."
							^ self
						].
				number <= min
					ifTrue: [number _ nil]
			].			
	self max: number! !

!ScaleMorph2 methodsFor: 'menu' stamp: 'mk 2/24/2004 23:33'!
changeMin
	"We expect an integer which is less than maxX."
	| string number convertBlock |
	[number isNil]
		whileTrue:
			[
				string _ FillInTheBlank
					request: 'Enter new minimal value.'
					initialAnswer: min asString.
				convertBlock _ [number _ string asNumber asInteger].
				convertBlock on: Error do: [].
				string = ''
					ifTrue:
						[
							"User pressed cancel."
							^ self
						].
				max <= number
					ifTrue: [number _ nil]
			].			
	self min: number! !

!ScaleMorph2 methodsFor: 'menu' stamp: 'mk 2/25/2004 14:16'!
changeMinorTick
	"We expect an integer which is not less than 1."
	| string number convertBlock |
	[number isNil]
		whileTrue:
			[
				string _ FillInTheBlank
					request: 'Enter new minor tick value.'
					initialAnswer: minorTick asString.
				convertBlock _ [number _ string asNumber asInteger].
				convertBlock on: Error do: [].
				string = ''
					ifTrue:
						[
							"User pressed cancel."
							^ self
						].
				number < 1
					ifTrue: [number _ nil]
			].
			
	self minorTick: number! !

!ScaleMorph2 methodsFor: 'menu' stamp: 'mk 4/30/2005 17:52'!
changeSubMajorTickFactor
	"We expect an integer which is not less than 2."
	| string number convertBlock |
	[number isNil]
		whileTrue:
			[
				string _ FillInTheBlank
					request: 'Enter new sub-major tick factor value.'
					initialAnswer: subMajorTickFactor asString.
				convertBlock _ [number _ string asNumber asInteger].
				convertBlock on: Error do: [].
					string = ''
					ifTrue:
						[
							"User pressed cancel."
							^ self
						].
				number < 1
					ifTrue: [number _ nil]
			].
			
	self subMajorTickFactor: number! !


!ScaleMorph2 methodsFor: 'menus' stamp: 'mk 2/24/2004 23:33'!
addCustomMenuItems: aCustomMenu hand: aHandMorph

	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	aCustomMenu add: 'caption' action: #changeCaption.
	aCustomMenu add: 'minimal value' action: #changeMin.
	aCustomMenu add: 'maximal value' action: #changeMax.
	aCustomMenu add: 'minor tick' action: #changeMinorTick.
	aCustomMenu add: 'sub-major tick factor' action: #changeSubMajorTickFactor.
	aCustomMenu add: 'major tick factor' action: #changeMajorTickFactor.
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ScaleMorph2 class
	instanceVariableNames: ''!

!ScaleMorph2 class methodsFor: 'class initialization' stamp: 'mk 2/24/2004 23:41'!
initialize
	ClipInset _ 1@1 corner: 1@1.
	MinorTickSize _ 5.
	SubMajorTickSize _ 10.
	MajorTickSize _ 20.
	TickColor _ Color black.
	LabelColor _ Color black.! !


ScaleMorph2 subclass: #XScaleMorph2
	instanceVariableNames: ''
	classVariableNames: 'VerticalCaptionSkip VerticalTextSkip'
	poolDictionaries: ''
	category: 'ScaleMorph2'!
!XScaleMorph2 commentStamp: 'mk 2/25/2004 17:47' prior: 0!
Horizontal version of the scale.!


!XScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/24/2004 23:36'!
caption: aString
	captionMorph _ aString asMorph.
	captionMorphWidth _ captionMorph width.
	captionMorphHeight _ captionMorph height.
	self changed.! !


!XScaleMorph2 methodsFor: 'drawing' stamp: 'mk 2/26/2004 12:03'!
drawOn: aCanvas
	| transform lineBeginning lineEnd stringMorphWidth stringMorphPosition stringMorph fictiveBounds clipRect offset1 scale offset2 captionMorphPosition |
	
	super drawOn: aCanvas.
	
	self updateLabels.
	clipRect _ bounds insetBy: ClipInset.
	aCanvas clipBy: clipRect during:
		[:clippedCanvas |
			"Fictive bounds of the drawing area, i.e.
			top-left point has (the smallest) coordinates minX@0
			and bottom-right point has (the biggest) coordinates maxX@100."
			fictiveBounds _ min @ 0 corner: max @ 100.
			
			offset1 _ MatrixTransform2x3 withOffset: (fictiveBounds topLeft negated).
			scale _ MatrixTransform2x3 withScale: (clipRect extent / fictiveBounds extent) asFloatPoint.
			offset2 _ MatrixTransform2x3 withOffset: (clipRect topLeft).
			transform _ offset2 composedWithLocal: (scale composedWithLocal: offset1).
				
			min to: max by: minorTick do:
				[:tick |
					lineBeginning _ transform localPointToGlobal: tick@0.
					clippedCanvas clipRect right <= lineBeginning x
						ifTrue:
							[
								"If the beginning of a line tick is on the right border
								of the clipping area or even right of it, it may happen that
								it will not be displayed. We move this point one point leftwards 
								from the right border of the clipping area."
								lineBeginning _ (clipRect right - 1) @ lineBeginning y.
							].
					lineBeginning x <= clippedCanvas clipRect left
						ifTrue:
							[
								"If the beginning of a line tick is on the left border
								of the clipping area or even left of it, it may happen that
								it will not be displayed. We move this point one point leftwards 
								from the right border of the clipping area."
								lineBeginning _ (clipRect left + 1) @ lineBeginning y.
							].
					(tick \\ subMajorTick = 0) not
						ifTrue:
							[
								"Compute minor tick line end."
								lineEnd _ lineBeginning + (0 @ MinorTickSize).
							].
					(tick \\ subMajorTick = 0) &
					(tick \\ majorTick ~= 0)
						ifTrue:
							[
								"Compute sub-major tick line end."
								lineEnd _ lineBeginning + (0 @ SubMajorTickSize)
							].
					(tick \\ majorTick = 0)
						ifTrue:
							[
								"Compute major tick line end."
								lineEnd _ lineBeginning + (0 @ MajorTickSize).
							].
					clippedCanvas
						line: lineBeginning
						to: lineEnd
						color: TickColor
				].
			
			"Draw the labels attached to the particular major ticks."
			majorTicks do:
				[:currentMajorTick |
					stringMorph _ labels at: currentMajorTick.
					stringMorphWidth _ stringMorph width.
					stringMorphPosition _ (transform localPointToGlobal: currentMajorTick@0) 
						+ ((stringMorphWidth / 2 negated) rounded
							@ (MajorTickSize + VerticalTextSkip)).
					stringMorph position: stringMorphPosition.
					stringMorph bounds: (stringMorph bounds translatedToBeWithin: clipRect).
					clippedCanvas drawMorph: stringMorph.
				].
			captionMorphPosition _ clipRect bottomCenter
				- (captionMorphWidth // 2
					@ (captionMorphHeight + VerticalCaptionSkip)).
			captionMorph position: captionMorphPosition.
			clippedCanvas drawMorph: captionMorph
		]! !


!XScaleMorph2 methodsFor: 'initialization' stamp: 'mk 2/26/2004 12:02'!
initialize
	super initialize.
	
	self caption: 'x scale'! !

!XScaleMorph2 methodsFor: 'initialization' stamp: 'mk 4/30/2005 19:05'!
updateLabels
	| numberOfMajorTicks i |

	subMajorTick _ minorTick * subMajorTickFactor.
	majorTick _ subMajorTick * majorTickFactor.

	"Fictive x coordinate of the left-most major tick."
	(min \\ majorTick = 0)
		ifTrue: [majorMin _ min]
		ifFalse: [majorMin _ min - (min \\ majorTick) + majorTick].

	"Fictive x coordinate of the right-most major tick."
	majorMax _ max - (max \\ majorTick).
	
	numberOfMajorTicks _ max // majorTick + 1 + (min negated // majorTick).
	majorTicks _ Array new: numberOfMajorTicks.
	labels _ Dictionary new.
	i _ 1.
	majorMin to: majorMax by: majorTick do:
		[:tick |
			tick asInteger = tick ifTrue: [tick _ tick asInteger].
			labels at: tick put: ((labelBlock value: tick) asMorph color: LabelColor).
			majorTicks at: i put: tick.
			i _ i + 1
		]! !


!XScaleMorph2 methodsFor: 'menu' stamp: 'mk 2/25/2004 00:01'!
changeCaption
	| string |
	string _ FillInTheBlank
				request: 'Enter new caption.'
				initialAnswer: captionMorph contents.
	string isEmpty not
		ifTrue: [self caption: string]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

XScaleMorph2 class
	instanceVariableNames: ''!

!XScaleMorph2 class methodsFor: 'class initialization' stamp: 'mk 2/24/2004 23:53'!
initialize
	VerticalTextSkip _ 5.
	VerticalCaptionSkip _ 5.! !


!XScaleMorph2 class methodsFor: 'examples' stamp: 'mk 2/25/2004 15:31'!
example1
	^ (self new extent: 400@60) openInWorld! !

!XScaleMorph2 class methodsFor: 'examples' stamp: 'mk 2/25/2004 15:31'!
example2
	^ (self new
		extent: 400@60;
		min: 100 max: 200) openInWorld! !

!XScaleMorph2 class methodsFor: 'examples' stamp: 'mk 2/25/2004 15:31'!
example3
	^ (self new
		extent: 400@60;
		min: 2/10 max: 6/10;
		minorTick: 1/100;
		labelBlock: [:i | i asFloat asString]) openInWorld! !


ScaleMorph2 subclass: #YScaleMorph2
	instanceVariableNames: ''
	classVariableNames: 'HorizontalTextSkip'
	poolDictionaries: ''
	category: 'ScaleMorph2'!
!YScaleMorph2 commentStamp: 'mk 2/25/2004 17:48' prior: 0!
Vertical version of the scale.!


!YScaleMorph2 methodsFor: 'accessing' stamp: 'mk 2/24/2004 23:58'!
caption: aString
	captionMorph _ aString asMorph.
	captionMorph _ TransformationMorph new asFlexOf: captionMorph.
	captionMorph angle: Float pi / 2.
	captionMorphHeight _ captionMorph height.
	self changed! !


!YScaleMorph2 methodsFor: 'drawing' stamp: 'mk 2/26/2004 12:03'!
drawOn: aCanvas
	| transform fictiveBounds lineBeginning lineEnd clipRect stringMorph stringMorphHeight stringMorphPosition stringMorphWidth rotation scale offset1 offset2 captionMorphPosition |
	
	super drawOn: aCanvas.
	self updateLabels.
	clipRect _ bounds insetBy: ClipInset.
	aCanvas clipBy: clipRect during:
		[:clippedCanvas |
			"Fictive bounds of the drawing area, i.e.
			bottom-right point has (the smallest) coordinates minY@0
			and top-left point has (the biggest) coordinates maxY@100."
			fictiveBounds _ 100 @ max corner: 0 @ min.

			offset1 _ MatrixTransform2x3 withOffset: (fictiveBounds topLeft negated).
			rotation _ MatrixTransform2x3 withRotation: 180.
			scale _ MatrixTransform2x3 withScale: (clipRect extent / fictiveBounds extent) abs asFloatPoint.
			offset2 _ MatrixTransform2x3 withOffset: (clipRect topLeft).
			transform _ offset2 composedWithLocal: (scale composedWithLocal: (rotation composedWithLocal: offset1)).
				
			min to: max by: minorTick do:
				[:tick |
					lineBeginning _ transform localPointToGlobal: 0@tick.
					clippedCanvas clipRect bottom <= lineBeginning y
						ifTrue:
							[
								"If the beginning of the line tick is on the bottom border
								of the clipping area or even below it,
								it may happen that it will not be displayed. We move this point
								one point upwards from the bottom border of the clipping area."
								lineBeginning _ lineBeginning x @ (clipRect bottom - 1)
							].
					
					lineBeginning y <= clippedCanvas clipRect top
						ifTrue:
							[
								"If the beginning of the line tick is on the top border
								of the clipping area or even above it,
								it may happen that it will not be displayed. We move this point
								one point downwards from the top border of the clipping area."
								lineBeginning _ lineBeginning x @ (clipRect top + 1)
							].
						
					(tick \\ subMajorTick = 0) not
						ifTrue:
							[
								"Compute minor tick line end."
								lineEnd _ lineBeginning - (MinorTickSize @ 0).
							].
						
					(tick \\ subMajorTick = 0) &
					(tick \\ majorTick ~= 0)
						ifTrue:
							[
								"Compute sub-major tick line end."
								lineEnd _ lineBeginning - (SubMajorTickSize @ 0).
							].
						
					(tick \\ majorTick = 0)
						ifTrue:
							[
								"Compute major tick line end."
								lineEnd _ lineBeginning - (MajorTickSize @ 0).
							].
						
					clippedCanvas
						line: lineBeginning
						to: lineEnd
						color: TickColor
				].
			"Draw the labels attached to the particular major ticks."
			majorTicks do:
				[:currentMajorTick |
					stringMorph _ labels at: currentMajorTick.
					stringMorphHeight _ stringMorph height.
					stringMorphWidth _ stringMorph width.
					stringMorphPosition _ (transform localPointToGlobal: 0@currentMajorTick) 
						+ ((MajorTickSize + HorizontalTextSkip + stringMorphWidth) negated
							@ (stringMorphHeight / 2 negated) rounded).
					stringMorph position: stringMorphPosition.
					stringMorph bounds: (stringMorph bounds translatedToBeWithin: clipRect).
					clippedCanvas drawMorph: stringMorph.
				].
			captionMorphPosition _ clipRect leftCenter
				+ (0 @ captionMorphHeight // 2 negated).
			captionMorph position: captionMorphPosition.
			captionMorph drawSubmorphsOn: clippedCanvas
		]! !


!YScaleMorph2 methodsFor: 'initialization' stamp: 'mk 2/26/2004 12:02'!
initialize
	super initialize.
	
	self caption: 'y scale'! !

!YScaleMorph2 methodsFor: 'initialization' stamp: 'mk 4/30/2005 18:59'!
updateLabels
	| numberOfMajorTicks i |
	subMajorTick _ minorTick * subMajorTickFactor.
	majorTick _ subMajorTick * majorTickFactor.

	"Fictive x coordinate of the left-most major tick."
	(min \\ majorTick = 0)
		ifTrue: [majorMin _ min]
		ifFalse: [majorMin _ min - (min \\ majorTick) + majorTick].

	"Fictive x coordinate of the right-most major tick."
	majorMax _ max - (max \\ majorTick).
	
	numberOfMajorTicks _ max // majorTick + 1 + (min negated // majorTick).
	majorTicks _ Array new: numberOfMajorTicks.
	labels _ Dictionary new.
	i _ 1.
	majorMin to: majorMax by: majorTick do:
		[:tick |
			tick asInteger = tick ifTrue: [tick _ tick asInteger].
			labels at: tick put: ((labelBlock value: tick) asMorph color: LabelColor).
			majorTicks at: i put: tick.
			i _ i + 1
		]
			! !


!YScaleMorph2 methodsFor: 'menu' stamp: 'mk 2/25/2004 00:10'!
changeCaption
	| string |
	string _ FillInTheBlank
				request: 'Enter new caption.'
				initialAnswer: (captionMorph submorphs at: 1) contents.
	string isEmpty not
		ifTrue: [self caption: string]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

YScaleMorph2 class
	instanceVariableNames: ''!

!YScaleMorph2 class methodsFor: 'class initialization' stamp: 'mk 2/24/2004 23:58'!
initialize
	HorizontalTextSkip _ 5! !


!YScaleMorph2 class methodsFor: 'examples' stamp: 'mk 2/25/2004 15:31'!
example1
	^ (self new extent: 70@400) openInWorld! !

!YScaleMorph2 class methodsFor: 'examples' stamp: 'mk 2/25/2004 15:31'!
example2
	^ (self new
		extent: 70@400;
		min: 100 max: 200) openInWorld! !

!YScaleMorph2 class methodsFor: 'examples' stamp: 'mk 2/25/2004 15:31'!
example3
	^ (self new
		extent: 70@400;
		min: 2/10 max: 6/10;
		minorTick: 1/100;
		labelBlock: [:i | i asFloat asString]) openInWorld! !

ScaleMorph2 initialize!
XScaleMorph2 initialize!
YScaleMorph2 initialize!